function [U,S,V,Ssq,ErrFlag] = mlpca_e(X,Cov,p);
%MLPCA_E Performs maximum likelihood principal components analysis for 
%        mode E error conditions (correlated errors, with a different
%        error covariance matrix for each row, but no error correlation
%        between the rows. Employs an ALS algorithm.
%
%   [U,S,V,SSQ,ERRFLAG] = MLPCA_E(X,COV,P) produces the MLPCA estimate of
%   the rank P subspace for the IxJ matrix X, given the measurement error 
%   covariance matrices in COV (JxJxI). Note that P must be less than the
%   minimum.
%   
%   The returned parameters, U(IxP), S(PxP) and V(JxP), are analogs to the
%   truncated SVD solution (SVDS(X,P)), but have somewhat different 
%   properties since they represent the MLPCA solution.  In particular, 
%   the solutions for different values of P are not necessarily nested
%   (the rank 1 solution may not be in the space of the rank 2 solution)
%   and the eigenvectors do not necessarily account for decreasing 
%   amounts of variance, since MLPCA is a subspace modeling technique and
%   not a variance modeling technique.  The parameters returned are the
%   results of SVD on the estimated subspace.  The quantity SSQ represents
%   the sum of squares of weighted residuals (objective function). ERRFLAG 
%   indicates the convergence condition, with 0 indicating normal 
%   termination and 1 indicating the maximum number of iterations have 
%   been exceeded.
%

% Section to check for valid inputs
%
if nargin~=3
    error('mlpca_e:err1','Incorrect number of arguments')
end
[m,n]=size(X);
if p>=min([m n])
    error('mlpca_e:err2','Invalid rank for MLPCA decomposition')
end
[m1,n1,q]=size(Cov);
if (n~=m1 | n~=n1 | m~=q)
    error('mlpca_c:err3','Invalid dimensions of covariance matrix')
end
% 
% Initialization
%
convlim=1e-10;             % convergence limit
maxiter=20000;            % maximum no. of iterations
%
% Calculate the inverse of the full covariance matrix blockwise
%
mn=m*n;
Q=spalloc(mn,mn,mn);
for i=1:m
    indx=(i-1)*n;
    Tmp=Cov(:,:,i);
    Q(indx+1:indx+n,indx+1:indx+n)=pinv(Tmp);
end
%
% Now find the commutation matrix for the covariance matrix and
% apply to Q.
%
ix=(1:mn)';
iy=reshape((reshape(ix,m,n))',mn,1);
K=sparse(ix,iy,1,mn,mn);
Q=K'*Q*K;
%
% Generate initial estimates assuming an average covariance matrix.
%
Covavg=mean(Cov,3); 
%
[U,S,V,Sobj]=mlcovav(X,Covavg,p);    % Initial guess
%
% Main loop to do alternating regression for MLPCA solution
%
count=0;
Sold=0;
ErrFlag=-1;
%
XX=X;
while ErrFlag<0;
   count=count+1;
%
% Vectorize X and create big U.  A Kronecker product is probably
% prettier for generating Ubig, but probably not as fast.
%
   vecX=reshape(XX,mn,1);
   Ubig=spalloc(mn,n*p,mn*p);
   for i=1:n
      indx1=(i-1)*m;
      indx2=(i-1)*p;
      Ubig(indx1+1:indx1+m,indx2+1:indx2+p)=U;
   end
%
% Evaluate objective function
%
   F=pinv(full(Ubig'*Q*Ubig));
   vecMLX=Ubig*(F*(Ubig'*(Q*vecX)));
   dx=vecX-vecMLX;
   Sobj=dx'*Q*dx;
   MLX=reshape(vecMLX,m,n);
%
% This section for diagnostics only and can be commented out.
%
%   Sobj
%   Ssave(count)=Sobj;
%   save mlcov;
%
% End diagnostics
%
% Check for convergence or excessive iterations
%
   if rem(count,2)==1                   % Check on odd iterations only
      convtst=(abs(Sold-Sobj)/abs(Sobj));
%       disp(count);                    % diagnostics
%       disp(Sobj);
%       disp(convtst)
      if (abs(Sold-Sobj)/abs(Sobj))<convlim  % Convergence criterion
         ErrFlag=0;
      elseif count>maxiter              % Excessive iterations?
         ErrFlag=1;
      end
   end
%
% Now flip matrices for alternating regression
%
   if ErrFlag<0                         % Only do this part if not finished
      Sold=Sobj;                        % Save most recent obj. function
      [U,S,V]=svd(MLX,0);               % Decompose ML values
      XX=XX';                           % Flip matrix
      Q=K*Q*K';
      K=K';
      m=length(XX(:,1));
      n=length(XX(1,:));              % Adjust no. of columns
      U=V(:,1:p);                      % V becomes U in for transpose
   end
end
%
% All done.  Clean up and go home.
%
[U,S,V]=svd(MLX,0);
U=U(:,1:p);
S=S(1:p,1:p);
V=V(:,1:p);
Ssq=Sobj;
%

function [U,S,V,Ssq]=mlcovav(X,Cov,p)
%MLCOVAV Performs maximum likelihood principal components analysis for 
%        mode D error conditions (common row covariance matrices). 
%        Employs rotation and scaling of the original data.
%
[m,n]=size(X);
df=(m-p)*(n-p);
[U1,S1,V1]=svd(Cov);
covrank=rank(Cov);      % If the covariance matrix is singular, then
if covrank<n            % expand uniformly in deficient directions
   scale=[sqrt(diag(S1(1:covrank,1:covrank)))'...
       ones(1,n-covrank)*sqrt(S1(covrank,covrank))*0.01];
else
   scale=sqrt(diag(S1));
end    
Z=X*U1*diag(1./scale);
[U2,S2,V2]=svd(Z);
Zcalc=U2(:,1:p)*S2(1:p,1:p)*V2(:,1:p)';
Ssq=0;
for i=1:m;
   Ssq=Ssq+norm(Zcalc(i,1:covrank)-Z(i,1:covrank))^2;
end
Xcalc=Zcalc*diag(scale)*U1';
[U,S,V]=svd(Xcalc);
U=U(:,1:p);
S=S(1:p,1:p);
V=V(:,1:p);
